/*
 * Copyright (c) 1999 Greg Haerr <greg@censoft.com>
 *
 * Keyboard Driver, RTEMS Version - Added code to console.c to 
 * handel the routines required by this module.
 */
#include <pthread.h>
#include <stdio.h>
#include <sys/types.h>
#include "../device.h

#define  RTEMS_KBD_EVENT   RTEMS_EVENT_19

static int  KBD_Open(KBDDEVICE *pkd);
static void KBD_Close(void);
static void KBD_GetModifierInfo(MODIFIER *modifiers);
static int  KBD_Read(UCHAR *buf, MODIFIER *modifiers);
static int	KBD_Poll(void);

/* external routines in asmkbd.s*/
extern int rtems_kbpoll(void);
extern int rtems_kbread(void);
extern int rtems_kbflags(void);

KBDDEVICE kbddev = {
	KBD_Open,
	KBD_Close,
	KBD_GetModifierInfo,
	KBD_Read,
	KBD_Poll
};

/*
 * Open the keyboard.
 */
static int
KBD_Open(KBDDEVICE *pkd)
{
   /* Register with the keyboard to receive one event when a key is  pressed */
   rtems_active_waiting_id( _Thread_Executing->Object.id, RTEMS_KBD_EVENT );
   printk( "Mouse: KBD ID active\n" );
	return 1;
}

/*
 * Close the keyboard.
 */
static void
KBD_Close(void)
{
  rtems_active_waiting_id( 0, 0 );
}

/*
 * Return the possible modifiers for the keyboard.
 */
static  void
KBD_GetModifierInfo(MODIFIER *modifiers)
{
	*modifiers = 0;			/* no modifiers available */
}

/*
 * This reads one keystroke from the keyboard, and the current state of
 * the mode keys (ALT, SHIFT, CTRL).  Returns -1 on error, 0 if no data
 * is ready, and 1 if data was read.  This is a non-blocking call.
 */
static int
KBD_Read(UCHAR *buf, MODIFIER *modifiers)
{
	/* wait until a char is ready*/
	if(!rtems_kbpoll())
		return 0;

	/* read keyboard shift status*/
	*modifiers = rtems_kbflags();

	/* read keyboard character*/
	*buf = rtems_kbread();

	if(*buf == 0x1b)			/* special case ESC*/
		return -2;
	return 1;
}

static int
KBD_Poll(void)
{
	return rtems_kbpoll();
}
